import kotlin.math.*

const val MOD = 998_244_353

inline class ModInt(val x:Int) {
    operator fun plus(other: ModInt) = ModInt((x + other.x).let { if (it >= MOD) it - MOD else it })
    operator fun minus(other: ModInt) = ModInt((x - other.x).let { if (it < 0) it + MOD else it })
    operator fun times(other: ModInt) = ModInt((x.toLong() * other.x % MOD).toInt())
    fun power(p_: Int) : ModInt {
        var a = this
        var res = ModInt(1)
        var p = p_
        while (p != 0) {
            if ((p and 1) == 1) res *= a
            a *= a
            p = p shr 1
        }
        return res
    }
    operator fun div(other: ModInt) = this * other.power(MOD - 2)
    override fun toString() = x.toString()
    fun toHumanString() : String {
        val lim = 100000
        val q = (1 until 100).firstOrNull { (x.toLong() * it % MOD).let { it <= lim || it >= MOD - lim } } ?: return "???[$x]"
        val a = x.toLong() * q % MOD
        if (a <= lim) return "$a/$q"
        else return "${a - MOD}/$q"
    }
}

inline class ModIntArray(private val storage:IntArray) {
    operator fun get(index: Int) = ModInt(storage[index])
    operator fun set(index: Int, value: ModInt) { storage[index] = value.x }
    val size get() = storage.size
}
inline fun ModIntArray(n: Int, init: (Int) -> ModInt) = ModIntArray(IntArray(n) { init(it).x })


fun getComps(g: List<IntArray>) : List<IntArray> {
    val tin = IntArray(g.size) { -1 }
    val tup = IntArray(g.size) { -1 }
    var timer = 0
    val stack = mutableListOf<Pair<Int, Int>>()
    val ans = mutableListOf<IntArray>()
    fun dfs(v: Int, p: Int) {
        tin[v] = timer++
        tup[v] = tin[v]
        for (u in g[v]) {
            if (u == p || tin[u] >= tin[v]) continue
            stack.add(v to u)
            if (tin[u] != -1) {
                tup[v] = minOf(tup[v], tin[u])
                continue
            }
            dfs(u, v)
            tup[v] = minOf(tup[v], tup[u])
            if (tup[u] >= tin[v]) {
                val x = mutableListOf<Int>()
                while (stack.last().first != v || stack.last().second != u) {
                    x.add(stack.last().first)
                    stack.removeLast()
                }
                if (x.size == 0) {
                    x.add(u)
                }
                x.add(stack.last().first)
                stack.removeLast()
                ans.add(x.toIntArray())
            }
        }
    }
    dfs(0, -1)
    return ans
}

fun dfsc(comps: List<IntArray>, inComps: List<IntArray>, c: Int, pv: Int) : ModIntArray {
    val ans = ModIntArray(inComps.size + 1) { ModInt(0) }
    val id = comps[c].indexOf(pv)
    for ((index, x) in comps[c].withIndex()) {
        if (x != pv) {
            val inner = dfsv(comps, inComps, x, c)
            for (i in 0 until ans.size - 1) {
                if (inner[i].x != 0) {
                    val part1 = abs(index - id)
                    val part2 = comps[c].size - part1
                    ans[i + part1 - 1] += inner[i]
                    ans[i + part2 - 1] += inner[i]
                    ans[i + part1 + part2 - 2] -= inner[i]
                }
            }
        }
    }
    return ans
}

fun dfsv(comps: List<IntArray>, inComps: List<IntArray>, v: Int, pc: Int) : ModIntArray {
    val ans = ModIntArray(inComps.size + 1) { ModInt(0) }
    ans[0] = ModInt(1)
    for (x in inComps[v]) {
        if (x != pc) {
            val inner = dfsc(comps, inComps, x, v)
            for (i in 0 until ans.size - 1) {
                ans[i + 1] += inner[i]
            }
        }
    }
    return ans
}

fun solve(comps: List<IntArray>, inComps: List<IntArray>, root: Int) : ModIntArray {
    return dfsv(comps, inComps, root, -1)
}

fun main() {
    val (n, m) = readInts()
    val g_ = List(n) { mutableListOf<Int>() }
    repeat(m) {
        val (a, b) = readInts().map { it - 1 }
        g_[a].add(b)
        g_[b].add(a)
    }
    val g = g_.map { it.toIntArray() }
    val comps = getComps(g)
    val inComps_ = List(n) { mutableListOf<Int>() }
    for (i in comps.indices) {
        for (j in comps[i]) {
            inComps_[j].add(i)
        }
    }
    val inComps = inComps_.map { it.toIntArray() }
    var ans = ModInt(0)
    for (i in 0 until n) {
        val r = solve(comps, inComps, i)
        //System.err.println(r)
        for (j in 0 until r.size) {
            ans += r[j] / ModInt(j + 1)
        }
    }
    //System.err.println(ans.toHumanString())
    println(ans.x)
}

private fun readLn() = readLine()!!
private fun readInt() = readLn().toInt()
private fun readLongs() = readStrings().map { it.toLong() }
private fun readStrings() = readLn().split(" ")
private fun readInts() = readStrings().map { it.toInt() }