/**
 *    author:  tourist
 *    created: 31.10.2021 02:06:45       
**/
#include <bits/stdc++.h>

using namespace std;

template <typename T>
class graph {
 public:
  struct edge {
    int from;
    int to;
    T cost;
  };

  vector<edge> edges;
  vector<vector<int>> g;
  int n;

  graph(int _n) : n(_n) {
    g.resize(n);
  }

  virtual int add(int from, int to, T cost) = 0;
};

template <typename T>
class forest : public graph<T> {
 public:
  using graph<T>::edges;
  using graph<T>::g;
  using graph<T>::n;

  forest(int _n) : graph<T>(_n) {
  }

  int add(int from, int to, T cost = 1) {
    assert(0 <= from && from < n && 0 <= to && to < n);
    int id = (int) edges.size();
    assert(id < n - 1);
    g[from].push_back(id);
    g[to].push_back(id);
    edges.push_back({from, to, cost});
    return id;
  }
};

template <typename T>
class dfs_forest : public forest<T> {
 public:
  using forest<T>::edges;
  using forest<T>::g;
  using forest<T>::n;

  vector<int> pv;
  vector<int> pe;
  vector<int> order;
  vector<int> pos;
  vector<int> end;
  vector<int> sz;
  vector<int> root;
  vector<int> depth;
  vector<T> dist;

  dfs_forest(int _n) : forest<T>(_n) {
  }

  void init() {
    pv = vector<int>(n, -1);
    pe = vector<int>(n, -1);
    order.clear();
    pos = vector<int>(n, -1);
    end = vector<int>(n, -1);
    sz = vector<int>(n, 0);
    root = vector<int>(n, -1);
    depth = vector<int>(n, -1);
    dist = vector<T>(n);
  }

  void clear() {
    pv.clear();
    pe.clear();
    order.clear();
    pos.clear();
    end.clear();
    sz.clear();
    root.clear();
    depth.clear();
    dist.clear();
  }

 private:
  void do_dfs(int v) {
    pos[v] = (int) order.size();
    order.push_back(v);
    sz[v] = 1;
    for (int id : g[v]) {
      if (id == pe[v]) {
        continue;
      }
      auto &e = edges[id];
      int to = e.from ^ e.to ^ v;
      depth[to] = depth[v] + 1;
      dist[to] = dist[v] + e.cost;
      pv[to] = v;
      pe[to] = id;
      root[to] = (root[v] != -1 ? root[v] : to);
      do_dfs(to);
      sz[v] += sz[to];
    }
    end[v] = (int) order.size() - 1;
  }

  void do_dfs_from(int v) {
    depth[v] = 0;
    dist[v] = T{};
    root[v] = v;
    pv[v] = pe[v] = -1;
    do_dfs(v);
  }

 public:
  void dfs(int v, bool clear_order = true) {
    if (pv.empty()) {
      init();
    } else {
      if (clear_order) {
        order.clear();
      }
    }
    do_dfs_from(v);
  }

  void dfs_all() {
    init();
    for (int v = 0; v < n; v++) {
      if (depth[v] == -1) {
        do_dfs_from(v);
      }
    }
    assert((int) order.size() == n);
  }
};

template <typename T>
class lca_forest : public dfs_forest<T> {
 public:
  using dfs_forest<T>::edges;
  using dfs_forest<T>::g;
  using dfs_forest<T>::n;
  using dfs_forest<T>::pv;
  using dfs_forest<T>::pos;
  using dfs_forest<T>::end;
  using dfs_forest<T>::depth;

  int h;
  vector<vector<int>> pr;

  lca_forest(int _n) : dfs_forest<T>(_n) {
  }

  inline void build_lca() {
    assert(!pv.empty());
    int max_depth = 0;
    for (int i = 0; i < n; i++) {
      max_depth = max(max_depth, depth[i]);
    }
    h = 1;
    while ((1 << h) <= max_depth) {
      h++;
    }
    pr.resize(n);
    for (int i = 0; i < n; i++) {
      pr[i].resize(h);
      pr[i][0] = pv[i];
    }
    for (int j = 1; j < h; j++) {
      for (int i = 0; i < n; i++) {
        pr[i][j] = (pr[i][j - 1] == -1 ? -1 : pr[pr[i][j - 1]][j - 1]);
      }
    }
  }

  inline bool anc(int x, int y) {
    return (pos[x] <= pos[y] && end[y] <= end[x]);
  }

  inline int go_up(int x, int up) {
    assert(!pr.empty());
    up = min(up, (1 << h) - 1);
    for (int j = h - 1; j >= 0; j--) {
      if (up & (1 << j)) {
        x = pr[x][j];
        if (x == -1) {
          break;
        }
      }
    }
    return x;
  }

  inline int lca(int x, int y) {
    assert(!pr.empty());
    if (anc(x, y)) {
      return x;
    }
    if (anc(y, x)) {
      return y;
    }
    for (int j = h - 1; j >= 0; j--) {
      if (pr[x][j] != -1 && !anc(pr[x][j], y)) {
        x = pr[x][j];
      }
    }
    return pr[x][0];
  }
};

namespace Kth {

static const int maxn = (1 << 17); // smallest power of 2 not less than 10^5
 
int coords[maxn];
int array[maxn];
int arrayp[maxn];
 
typedef struct node {
    node *l, *r;
    int sum;
} *Node;
 
node buffer[maxn * 20]; // XXX: Perhaps 20 is not a correct value
Node ver[maxn];
 
Node allocate(int number = 1)
{
    static Node freep = buffer;
    if (freep + number > buffer + maxn * 1000)
        for (;;) ;
    Node res = freep;
    freep += number;
    return res;
}
 
Node update(Node v, int vl, int vr, int x)
{
    if (vl == x && vr == x + 1) {
        Node u = allocate();
        *u = *v;
        u->sum = 1;
        return u;
    }
    Node u = allocate();
    *u = *v;
    int m = (vl + vr) / 2;
    if (x < m)
        u->l = update(u->l, vl, m, x);
    else
        u->r = update(u->r, m, vr, x);
    ++u->sum;
    return u;
}
 
 
std::pair<int, bool> find_nth(Node u, Node v, int vl, int vr, int k)
{
    int sum = v->sum - u->sum;
    if (vr - vl == 1) {
        if (k == sum)
            return std::make_pair(vl, false);
    }
    if (sum < k)
        return std::make_pair(vr, true);
    if (vr - vl == 1)
        return std::make_pair(vl, false);
    auto p = find_nth(u->l, v->l, vl, (vl + vr) / 2, k);
    if (!p.second)
        return p;
    return find_nth(u->r, v->r, (vl + vr) / 2, vr, k - v->l->sum + u->l->sum);
}
 
void init(int n, vector<int> arr)
{
    for (int i = 0; i < n; ++i)
      array[i] = arr[i];
    std::copy(array, array + n, coords);
    std::sort(coords, coords + n);
    for (int i = 0; i < n; ++i)
        array[i] = std::lower_bound(coords, coords + n, array[i]) - coords;
    for (int  i = 0; i < n; ++i)
        arrayp[array[i]] = i;
    allocate(2 * maxn - 1);
    for (int i = maxn - 2; i >= 0; --i)  {
        buffer[i].l = &buffer[2 * i + 1];
        buffer[i].r = &buffer[2 * i + 2];
    }
    ver[0] = buffer;
    for (int i = 0; i < n; ++i)
        ver[i + 1] = update(ver[i], 0, maxn, array[i]);
}

int query(int l, int r, int k) {
    r += 1;
    int ret = find_nth(ver[l], ver[r], 0, maxn, k).first;
    return ret;
}

}

int main() {
  ios::sync_with_stdio(false);
  cin.tie(0);
  int n, y, q;
  cin >> n >> y >> q;
  lca_forest<int> g(n);
  for (int i = 0; i < n - 1; i++) {
    int xx, yy;
    cin >> xx >> yy;
    --xx; --yy;
    g.add(xx, yy);
  }
  g.dfs(0);
  g.build_lca();
  Kth::init(n, g.order);
  auto GetKth = [&](int ver, int k) {
    int lol = Kth::query(g.pos[ver], g.end[ver], k + 1);
    return lol;
  };
  vector<pair<long long, int>> seq;
  seq.emplace_back(0, 0);
  lca_forest<long long> gt(y + 1);
  vector<int> on(y + 1, -1);
  for (int i = 0; i < y; i++) {
    int f;
    long long t;
    cin >> f >> t;
    --f; --t;
    int id = (int) (lower_bound(seq.begin(), seq.end(), make_pair(t + 1, -1)) - seq.begin()) - 1;
    int k = (int) (t - seq[id].first);
    int me = GetKth(seq[id].second, k);
    on[i + 1] = me;
    gt.add(id, i + 1, g.depth[me] - g.depth[seq[id].second] + 1);
    seq.emplace_back(seq.back().first + g.sz[seq.back().second], f);
  }
  gt.dfs(0);
  gt.build_lca();
  while (q--) {
    long long xx, yy;
    cin >> xx >> yy;
    --xx; --yy;
    int id_x = (int) (lower_bound(seq.begin(), seq.end(), make_pair(xx + 1, -1)) - seq.begin()) - 1;
    int k_x = (int) (xx - seq[id_x].first);
    int me_x = GetKth(seq[id_x].second, k_x);
    int id_y = (int) (lower_bound(seq.begin(), seq.end(), make_pair(yy + 1, -1)) - seq.begin()) - 1;
    int k_y = (int) (yy - seq[id_y].first);
    int me_y = GetKth(seq[id_y].second, k_y);
    long long ans = 0;
    ans += gt.dist[id_x] + (g.depth[me_x] - g.depth[seq[id_x].second]);
    ans += gt.dist[id_y] + (g.depth[me_y] - g.depth[seq[id_y].second]);
    int id_z = gt.lca(id_x, id_y);
    int v_x = (id_x == id_z ? me_x : on[gt.go_up(id_x, gt.depth[id_x] - gt.depth[id_z] - 1)]);
    int v_y = (id_y == id_z ? me_y : on[gt.go_up(id_y, gt.depth[id_y] - gt.depth[id_z] - 1)]);
    int me_z = g.lca(v_x, v_y);
    ans -= 2 * (gt.dist[id_z] + (g.depth[me_z] - g.depth[seq[id_z].second]));
    cout << ans << '\n';
  }
  return 0;
}
