import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Random;
import java.util.StringTokenizer;

public class I_sm {
    class Cell {
        int sx, sy, x, y;

        public Cell(int x, int y) {
            this.sx = this.x = x;
            this.sy = this.y = y;
        }

        public Cell(int sx, int sy, int x, int y) {
            this.sx = sx;
            this.sy = sy;
            this.x = x;
            this.y = y;
        }

        boolean isValid() {
            return x >= 0 && x < 8 && y >= 0 && y < 8;
        }

        Cell go(int dx, int dy) {
            Cell c = new Cell(sx, sy, x + dx, y + dy);
            if (!c.isValid()) {
                return null;
            }
            return c;
        }
    }

    ArrayList<Cell> apply(ArrayList<Cell> src, int dx, int dy) {
        ArrayList<Cell> res = new ArrayList<>();
        for (Cell c : src) {
            Cell nc = c.go(dx, dy);
            if (nc != null) {
                res.add(nc);
            }
        }
        return res;
    }

    ArrayList<Cell> notApply(ArrayList<Cell> src, int dx, int dy) {
        ArrayList<Cell> res = new ArrayList<>();
        for (Cell c : src) {
            Cell nc = c.go(dx, dy);
            if (nc == null) {
                res.add(c);
            }
        }
        return res;
    }

    void print(ArrayList<Cell> a) {
        char[][] c = new char[8][8];
        for (int i = 0; i < 8; i++) {
            for (int j = 0; j < 8; j++) {
                c[i][j] = '.';
            }
        }
        for (Cell cell : a) {
            c[cell.sx][cell.sy] = 'X';
        }
        for (int i = 7; i >= 0; i--) {
            System.err.println(c[i]);
        }
        System.err.println();
    }

    void solve() {
        ArrayList<Cell> all = new ArrayList<>();
        for (int i = 0; i < 8; i++) {
            for (int j = 0; j < 8; j++) {
                all.add(new Cell(i, j));
            }
        }
        ArrayList<int[]> turns = new ArrayList<>();
        for (int dx = -2; dx <= 2; dx++) {
            for (int dy = -2; dy <= 2; dy++) {
                if (Math.abs(dx * dy) != 2) {
                    continue;
                }
                turns.add(new int[]{dx, dy});
            }
        }
        Random rand = new Random(0);
        int cnt = 0;
        while (true) {
            int best = all.size() + 1;
            int bestdx = 0;
            int bestdy = 0;
            Collections.shuffle(turns, rand);
            for (int[] turn : turns) {
                int dx = turn[0];
                int dy = turn[1];
                if (Math.abs(dx * dy) != 2) {
                    continue;
                }
                ArrayList<Cell> na = apply(all, dx, dy);
                int q = Math.max(na.size(), all.size() - na.size());
                if (q < best) {
                    best = q;
                    bestdx = dx;
                    bestdy = dy;
                }

            }
            out.println("? " + bestdx + " " + bestdy);
            cnt++;
            out.flush();
            if (next().equals("+")) {
                all = apply(all, bestdx, bestdy);
            } else {
                all = notApply(all, bestdx, bestdy);
            }
            print(all);
            if (all.size() == 1) {
                System.err.println(cnt);
                out.println("! " + (char) ('a' + all.get(0).sx) + (all.get(0).sy + 1));
                out.flush();
                return;
            }
        }

    }


    BufferedReader br;
    StringTokenizer st;
    PrintWriter out;

    String next() {
        while (st == null || !st.hasMoreTokens()) {
            try {
                st = new StringTokenizer(br.readLine());
            } catch (IOException e) {
                e.printStackTrace();
                System.exit(1);
            }
        }
        return st.nextToken();
    }

    int nextInt() {
        return Integer.parseInt(next());
    }

    void run() {
        br = new BufferedReader(new InputStreamReader(System.in));
        out = new PrintWriter(System.out);
        solve();
        out.close();
    }

    public static void main(String[] args) {
        new I_sm().run();
    }
}
