#include "testlib.h"

using namespace std;

int played[10][10];

vector <int> gms;
vector <int> cur;

int testID=4;


void clear_played ()
{
 for (int i=0;i<10;i++)
  for (int j=0;j<10;j++)
     played[i][j] = (i==j) ? 1 : 0;
}


void init_triples ()
{
   for (int i=0;i<9;i++)
    for (int j=0;j<i;j++)
     for (int k=0;k<j;k++)
        gms.push_back ( (1<<i) + (1<<j) + (1<<k) );
}

void decode_mask (int mask, int *arr)
{
  int j=0;
  for (int i=0;(i<9)&&(j<3);i++)
   {
     if (mask&1) arr[j++]=i+1;
     mask>>=1;
   }
}

int encode_mask (int x, int y, int z)
{
   return (1<<(x-1)) + (1<<(y-1)) + (1<<(z-1));
}


bool check_mask (int mask)
{
  int a[3];
  decode_mask (mask,a);
  for (int i=0;i<3;i++)
   if (played[a[i]][a[(i+1)%3]]) return false;
  return true;
}

void apply_mask (int mask)
{
  int a[3];
  decode_mask (mask,a);
  for (int i=0;i<3;i++)
   played[a[i]][a[(i+1)%3]]=played[a[(i+1)%3]][a[i]]=1;
}

void write_test (int n, vector <int> data)
{
  char str[10];
  int a[3];
  sprintf (str,"%03d",testID++);
  FILE *st1=fopen (str,"w");
  fprintf (st1,"%d\n",n);
  for (int i=0;i<n;i++)
   {
    decode_mask (data[i],a);
    shuffle (a,a+3);
    fprintf (st1,"%d %d %d\n",a[0],a[1],a[2]);
   }
  fclose (st1);
}

void gen_ok (int layer1, int layer2=-1, int layer3=-1, int layer4=-1)
{
 int n;
 vector <int> super9;
 vector <int> data;
 int pl[9];
 for (int i=0;i<9;i++)
  pl[i]=i+1;
 shuffle(pl,pl+9);
 super9.push_back (encode_mask (pl[0],pl[1],pl[2]));
 super9.push_back (encode_mask (pl[3],pl[4],pl[5]));
 super9.push_back (encode_mask (pl[6],pl[7],pl[8]));

 super9.push_back (encode_mask (pl[0],pl[3],pl[6]));
 super9.push_back (encode_mask (pl[1],pl[4],pl[7]));
 super9.push_back (encode_mask (pl[2],pl[5],pl[8]));

 super9.push_back (encode_mask (pl[0],pl[4],pl[8]));
 super9.push_back (encode_mask (pl[6],pl[1],pl[5]));
 super9.push_back (encode_mask (pl[3],pl[7],pl[2]));

 super9.push_back (encode_mask (pl[0],pl[7],pl[5]));
 super9.push_back (encode_mask (pl[1],pl[3],pl[8]));
 super9.push_back (encode_mask (pl[6],pl[4],pl[2]));

 if (layer2==-1) // random
 {
  n=layer1;
  shuffle (super9.begin(),super9.end());
  write_test (n,super9);
 }
 else
 {
  n=layer1+layer2+layer3+layer4;
  for (int i=0;i<layer1;i++)
   data.push_back (super9[i]);
  for (int i=0;i<layer2;i++)
   data.push_back (super9[i+3]);
  for (int i=0;i<layer3;i++)
   data.push_back (super9[i+6]);
  for (int i=0;i<layer4;i++)
   data.push_back (super9[i+9]);
  write_test (n,data);
 }
} 

void gen_rand (int n)
{
   shuffle (gms.begin(),gms.end());
   write_test (n,gms);
}

void gen_greedy_rand ()
{
 vector <int> res;
 shuffle (gms.begin(),gms.end());
 clear_played ();
 for (int i=0;i<gms.size();i++)
  {
   if (check_mask(gms[i]))
    {
      res.push_back (gms[i]);
      apply_mask (gms[i]);
    }
  } 
 write_test (res.size(),res);
}

void gen_greedy_fail ()
{
 int x,y,z;
 vector <int> res;
 clear_played();
  for (x=1;x<=9;x++)
   for (y=1;y<=9;y++)
    for (z=1;z<=9;z++)
     {
       if ( (played[x][y]) || (played[y][z]) || (played[x][z]) ) continue;
       played[x][y]=played[y][x]=1;
       played[z][y]=played[y][z]=1;
       played[x][z]=played[z][x]=1;
       res.push_back (encode_mask(x,y,z));
     }
  write_test (res.size(),res);
}


int main (int argc, char **argv)
{
 registerGen (argc,argv,1);
 init_triples ();
 fprintf (stderr,"%d\n",gms.size());

// rand

 gen_rand (9);
 gen_rand (10);
 gen_rand (11);
 gen_rand (12);
 gen_rand (13);
 gen_rand (20);
 gen_rand (21);
 gen_rand (30);
 gen_rand (gms.size());
 
// ok's
 
 for (int i=1;i<4;i++)
  for (int j=0;j<=i;j++)
   for (int k=0;k<=j;k++)
    for (int l=0;l<=k;l++)
     gen_ok (i,j,k,l);

 gen_ok (2);
 gen_ok (3);
 gen_ok (4);
 gen_ok (5);
 gen_ok (6);
 gen_ok (7);
 gen_ok (8);
 gen_ok (9);
 gen_ok (10);
 gen_ok (11);
 gen_ok (12);

 gen_greedy_fail ();

 for (int i=0;i<20;i++)
  gen_greedy_rand();
 

 return 0;
}

